package com.ccwangluo.ccim.manage;

import android.text.TextUtils;

import com.ccwangluo.ccim.CCIMClient;
import com.ccwangluo.ccim.modle.IrequestData;
import com.ccwangluo.ccim.modle.MyIReaderProtocol;
import com.ccwangluo.ccim.listener.MessageListener;
import com.ccwangluo.ccim.listener.ResultEventCallback;
import com.xuhao.didi.core.pojo.OriginalData;
import com.xuhao.didi.socket.client.sdk.OkSocket;
import com.xuhao.didi.socket.client.sdk.client.ConnectionInfo;
import com.xuhao.didi.socket.client.sdk.client.OkSocketOptions;
import com.xuhao.didi.socket.client.sdk.client.action.SocketActionAdapter;
import com.xuhao.didi.socket.client.sdk.client.connection.IConnectionManager;

import java.util.logging.Logger;

import ccim.protocol.Auth;
import ccim.protocol.Chat;
import ccim.protocol.Common;
import ccim.protocol.Message;
import ccim.protocol.Push;

public class SocketManageer {
    private final Logger logger = Logger.getLogger(CCIMClient.class.getName());

    private final int version = 1;

    private volatile int id = 1;

    private final CallBackManager<Chat.ChatResponseMessage> callBackManager;
    public SocketManageer(CallBackManager<Chat.ChatResponseMessage> callBackManager) {
        this.callBackManager = callBackManager;
    }

    private ConnectionInfo authInfo;
    private ConnectionInfo chatInfo;
    private String token;

    private ResultEventCallback<String> loginCallBack;

    public String getToken() {
        return token;
    }

    public void setToken(String token) {
        this.token = token;
    }

    public ConnectionInfo getAuthInfo() {
        return authInfo;
    }


    public ConnectionInfo getChatInfo() {
        return chatInfo;
    }


    public void initAuth() {
        //连接参数设置(IP,端口号),这也是一个连接的唯一标识,不同连接,该参数中的两个值至少有其一不一样
        this.authInfo = initServer("117.144.156.2", 9000, data -> {
            try {
                Auth.AuthResponseMessage chatResponseMessage = Auth.AuthResponseMessage.parseFrom(data.getBodyBytes());
                logger.info(String.valueOf(chatResponseMessage));
                if (chatResponseMessage.getHeader().getCode() == 0) {
                    token = chatResponseMessage.getLoginRes().getToken();
                    String server = chatResponseMessage.getLoginRes().getServer();
                    initChat(server);
                    OkSocket.open(authInfo).disconnect();
                    if (loginCallBack != null){
                        loginCallBack.onSuccess(token);
                    }
                } else {
                    if (loginCallBack != null){
                        loginCallBack.onFailed(chatResponseMessage.getHeader().getCode());
                    }
                }

            } catch (Exception e) {
                e.printStackTrace();
            }
        });
    }

    public void login(String username, String password,Long appId, ResultEventCallback<String> resultEventCallback) {
        Auth.LoginRequest.Builder builder = Auth.LoginRequest.newBuilder().setUsername(username).setPassword(password).setAppId(appId);
        Auth.AuthRequestMessage build = Auth.AuthRequestMessage.newBuilder().setLoginReq(builder.build()).build();
        OkSocket.open(getAuthInfo())
                .send(new IrequestData(build.toByteArray()));
        this.loginCallBack = resultEventCallback;
    }

    public void connectChatServer() {
        if (TextUtils.isEmpty(getToken())) {
            return;
        }
        Chat.ConnectRequest builder = Chat.ConnectRequest.newBuilder().setToken(getToken()).build();
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(1).setSeq(1).build();
        Chat.ChatRequestMessage build = Chat.ChatRequestMessage.newBuilder().setConnectReq(builder).setHeader(header).build();
        OkSocket.open(getChatInfo())
                .send(new IrequestData(build.toByteArray()));
    }

    private void initChat(String server) {
        //连接参数设置(IP,端口号),这也是一个连接的唯一标识,不同连接,该参数中的两个值至少有其一不一样
        String[] split = server.split(":");
        this.chatInfo  = initServer(split[0], Integer.parseInt(split[1]), new MessageListener() {
            @Override
            public void onServerConnect() {
                connectChatServer();
            }

            @Override
            public void onMessageListener(OriginalData data) {
                try {
                    Chat.ChatResponseMessage chatResponseMessage = Chat.ChatResponseMessage.parseFrom(data.getBodyBytes());
                    int seq = chatResponseMessage.getHeader().getSeq();
                    ResultEventCallback<Chat.ChatResponseMessage> callBack = callBackManager.getCallBack(seq);
                    if (callBack != null) {
                        if (seq > 0) {
                            if (chatResponseMessage.getHeader().getCode() == 0) {
                                callBack.onSuccess(chatResponseMessage);
                            } else {
                                callBack.onFailed(chatResponseMessage.getHeader().getCode());
                            }
                            if (!chatResponseMessage.hasPullOfflineMsgRes()) {
                                callBackManager.removeCallBack(seq);
                            }
                        }
                        else if (seq == 0) { //push消息
                            callBack.onSuccess(chatResponseMessage);
                        }
                    }

                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });
    }

    private ConnectionInfo initServer(String ip, int port, MessageListener messageListener) {
        ConnectionInfo authInfo = new ConnectionInfo(ip, port);
        //调用OkSocket,开启这次连接的通道,拿到通道Manager
        IConnectionManager manager = OkSocket.open(authInfo);
        //获得当前连接通道的参配对象
        OkSocketOptions options = manager.getOption();
        //基于当前参配对象构建一个参配建造者类
        OkSocketOptions.Builder builder = new OkSocketOptions.Builder(options);
        //修改参配设置(其他参配请参阅类文档)
        //        builder.setSinglePackageBytes(size);
        //建造一个新的参配对象并且付给通道
        manager.option(builder.build());
        //调用通道进行连接
        manager.connect();

        //设置自定义解析头
        OkSocketOptions.Builder okOptionsBuilder = new OkSocketOptions.Builder();
        okOptionsBuilder.setReaderProtocol(new MyIReaderProtocol());
        //将新的修改后的参配设置给连接管理器
        manager.option(okOptionsBuilder.build());

        manager.registerReceiver(new SocketActionAdapter() {
            @Override
            public void onSocketConnectionSuccess(ConnectionInfo info, String action) {
                super.onSocketConnectionSuccess(info, action);
                logger.info("connect success");
                messageListener.onServerConnect();

            }

            @Override
            public void onSocketDisconnection(ConnectionInfo info, String action, Exception e) {
                super.onSocketDisconnection(info, action, e);
                logger.info("disconnect success");
            }

            @Override
            public void onSocketReadResponse(ConnectionInfo info, String action, OriginalData data) {
                super.onSocketReadResponse(info, action, data);
                messageListener.onMessageListener(data);
            }
        });
        return authInfo;
    }

    public synchronized int sendRequest(Object request) {
        Common.RequestMessageHeader header = Common.RequestMessageHeader.newBuilder().setVersion(version).setSeq(id).build();
        Chat.ChatRequestMessage.Builder builder  = Chat.ChatRequestMessage.newBuilder();
        builder.setHeader(header);
        if (request instanceof Chat.PullContactsRequest){
            builder.setPullContactsReq((Chat.PullContactsRequest)request);
        }else if (request instanceof Chat.SearchRequest){
            builder.setSearchReq((Chat.SearchRequest)request);
        }else if (request instanceof Chat.AddContactRequest){
            builder.setAddContactReq((Chat.AddContactRequest)request);
        } else if (request instanceof Chat.ContactRequestsRequest){
            builder.setContactRequestsReq((Chat.ContactRequestsRequest) request);
        } else if(request instanceof Chat.CheckContactReqRequest){
            builder.setCheckContactReqReq((Chat.CheckContactReqRequest) request);
        } else if (request instanceof Chat.CreateContactGroupRequest){
            builder.setCreateContactGroupReq((Chat.CreateContactGroupRequest) request);
        } else if (request instanceof Chat.MoveContactGroupRequest){
            builder.setMoveContactGroupReq((Chat.MoveContactGroupRequest) request);
        } else if (request instanceof Chat.DeleteContactRequest){
            builder.setDeleteContactReq((Chat.DeleteContactRequest) request);
        } else if (request instanceof Chat.BlockContactRequest){
            builder.setBlockContactReq((Chat.BlockContactRequest) request);
        } else if (request instanceof Chat.ReportIllegalUserRequest){
            builder.setReportIllegalUserReq((Chat.ReportIllegalUserRequest) request);
        } else if (request instanceof Message.SendMessageRequest){
            builder.setSendMsgReq((Message.SendMessageRequest) request);
        } else if (request instanceof Message.SendGroupMessageRequest){
            builder.setSendGroupMsgReq((Message.SendGroupMessageRequest) request);
        } else if (request instanceof Message.SendChannelMessageRequest){
            builder.setSendChannelMsgReq((Message.SendChannelMessageRequest) request);
        } else if (request instanceof Push.PushPrivateMessageAckRequest){
            builder.setPushPrivateMsgAckReq((Push.PushPrivateMessageAckRequest) request);
        } else if (request instanceof Push.PushGroupMessageAckRequest){
            builder.setPushGroupMsgAckReq((Push.PushGroupMessageAckRequest) request);
        } else if (request instanceof Push.PushChannelMessageAckRequest){
            builder.setPushChannelMsgAckReq((Push.PushChannelMessageAckRequest) request);
        } else if (request instanceof Message.PullOfflineMessageRequest){
            builder.setPullOfflineMsgReq((Message.PullOfflineMessageRequest) request);
        }

        OkSocket.open(getChatInfo())
                .send(new IrequestData(builder.build().toByteArray()));

        return id++;
    }

}
