package com.ccwangluo.ccim.util;

import android.media.MediaRecorder;
import android.os.Build;

import java.io.File;
import java.io.IOException;

public class MediaHelper {
    private MediaRecorder mMediaRecorder;
    private String mPath;//文件夹
    private String mFilePath;//文件

    private static MediaHelper mInstance;

    private MediaHelper(String path) {
        mPath = path;
    }

    /**
     * 准备播放后的回调
     * 这个时候文件夹里已经有文件生成了
     * 如果不去释放资源将会一直进行录音
     */
    public interface MediaStateListener {
        void preparedDone();
    }

    public MediaStateListener mMediaStateListener;

    public void setMediaStateListener(MediaStateListener mediaStateListener) {
        mMediaStateListener = mediaStateListener;
    }

    /**
     * 单例模式获取 MediaHelper
     * 双检锁/双重校验锁
     *
     * @param path
     * @return
     */
    public static MediaHelper getInstance(String path) {
        if (mInstance == null) {
            synchronized (MediaHelper.class) {
                if (mInstance == null) {
                    mInstance = new MediaHelper(path);
                }
            }
        }
        return mInstance;
    }
    /**
     * 准备录音
     */
    public void prepare() {
        try {
            File fileDir = new File(mPath);
            boolean b = !fileDir.exists();
            if (b) {
                fileDir.mkdirs();
            }
            String fileName = System.currentTimeMillis() + ".amr"; // 文件名字
            File file = new File(fileDir, fileName);  // 文件路径
            mMediaRecorder = new MediaRecorder();
            mFilePath = file.getAbsolutePath();
            //设置保存文件的路径
            if (Build.VERSION.SDK_INT < 26) {
                //若api低于26，调用setOutputFile(String path)
                mMediaRecorder.setOutputFile(file.getAbsolutePath());
            } else {
                //若API高于26 使用setOutputFile(File path)
                mMediaRecorder.setOutputFile(new File(file.getAbsolutePath()));
            }
            mMediaRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);    // 设置MediaRecorder的音频源为麦克风
            mMediaRecorder.setOutputFormat(MediaRecorder.OutputFormat.AMR_NB);    // 设置音频的格式
            mMediaRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AMR_NB);    // 设置音频的编码为AMR_NB
            mMediaRecorder.prepare();
            mMediaRecorder.start();
            if (mMediaStateListener != null) {
                mMediaStateListener.preparedDone();
            }
        } catch (IllegalStateException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    /**
     * 释放资源
     */
    public void release() {
        mMediaRecorder.stop();
        mMediaRecorder.release();
        mMediaRecorder = null;
    }
    /**
     * 取消
     */
    public void cancel() {
        release();
        //删除相应的录音
        if (mFilePath != null) {
            File file = new File(mFilePath);
            if (file.exists()) {
                file.delete();
            }
            mFilePath = null;
        }
    }
    //获取生成的文件路径
    public String getFilePath() {
        return mFilePath;
        }
}